"""
This type stub file was generated by pyright.
"""

from .exceptions_types import EmailNotValidError
from .exceptions_types import EmailSyntaxError
from .exceptions_types import EmailUndeliverableError
from .exceptions_types import ValidatedEmail
from .validate_email import validate_email
from .version import __version__

__all__ = ['validate_email', 'ValidatedEmail', 'EmailNotValidError', 'EmailSyntaxError', 'EmailUndeliverableError', 'caching_resolver', '__version__']
def caching_resolver(*args, **kwargs): # -> Resolver:
    ...

ALLOW_SMTPUTF8 = ...
ALLOW_QUOTED_LOCAL = ...
ALLOW_DOMAIN_LITERAL = ...
GLOBALLY_DELIVERABLE = ...
CHECK_DELIVERABILITY = ...
TEST_ENVIRONMENT = ...
DEFAULT_TIMEOUT = ...
SPECIAL_USE_DOMAIN_NAMES = ...
